#ifndef GAME_H
#define GAME_H

#include "client/gui/StartGUI.h"
#include "client/input/Controller.h"
#include "client/rendering/RenderSettings.h"
#include "client/rendering/Renderer.h"
#include "client/rendering/ShaderMatrix.h"
#include "common/block/BlockRegistry.h"
#include "common/world/World.h"
#include "input/TextInput.h"
#include "network/Client.h"
#include "rendering/renderer/WorldRenderer.h"
#include "utils/Clock.h"
#include "utils/Size.h"

class Game final {
    const Controller& controller;
    const Clock& fps;
    const Clock& tps;
    RenderSettings& renderSettings;
    const Size& size;
    Client& client;

    typedef void (Game::*TickState)();
    TickState tickState;
    typedef void (Game::*RenderState)(float, ShaderMatrix&, Renderer&);
    RenderState renderState;
    BaseGUI baseGUI;
    StartGUI startGUI;

    Vector3 lastPos;
    Vector3 pos;
    Quaternion lastRotation;
    Quaternion rotation;

    World world;
    WorldRenderer worldRenderer;

public:
    static BlockRegistry blockRegistry;

    Game(TextInput*& textInput, const Controller& controller, const Clock& fps,
         const Clock& tps, RenderSettings& renderSettings, const Size& size,
         Client& client);

    void tick();
    void renderWorld(float lag, ShaderMatrix& sm);
    void renderOverlay(float lag, ShaderMatrix& sm, Renderer& r);

    bool isRunning() const;

    void onConnect();
    void onDisconnect();
    void onPacket(InPacket& in);

private:
    void tickConnectState();
    void tickConnectedState();
    void renderConnectState(float lag, ShaderMatrix& sm, Renderer& r);
    void renderConnectedState(float lag, ShaderMatrix& sm, Renderer& r);
};

#endif