#include "client/input/Controller.h"
#include "client/rendering/Engine.h"
#include "client/rendering/Framebuffers.h"
#include "client/rendering/RenderSettings.h"
#include "client/rendering/Shaders.h"
#include "network/Client.h"
#include "rendering/Window.h"
#include "utils/Logger.h"
#include "wrapper/GL.h"

int main() {
    Client client;
    Error error = client.start();
    if(error.has()) {
        LOG_ERROR(error.message);
        return 0;
    }
    WindowOptions options(4, 0, {1024, 620}, false, "test");
    Window w;
    error = w.open(options);
    if(error.has()) {
        LOG_ERROR(error.message);
        return 0;
    }

    Shaders shaders;
    error = shaders.init();
    if(error.has()) {
        LOG_ERROR(error.message);
        return 0;
    }

    Framebuffers framebuffers;
    error = framebuffers.init(w.getSize());
    if(error.has()) {
        LOG_ERROR(error.message);
        return 0;
    }

    RenderSettings renderSettings;
    Controller controller(w.buttons);
    Game game(w.textInput, controller, w.getFrameClock(), w.getTickClock(),
              renderSettings, w.getSize(), client);
    Engine engine(shaders, framebuffers, w.getSize(), renderSettings, game);
    w.run(engine, 50'000'000);
    return 0;
}
